
# q-expansion of newform 1080.2.q.e, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1080
weight = 2
poly_data = [4, 0, 32, 0, 36, 0, 11, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[2, 10, 0, 7, 0, 1, 0, 0], 4], [[2, -4, 10, -7, 7, -1, 1, 0], 4], [[2, 4, 10, 7, 7, 1, 1, 0], 4], [[0, 10, 18, -8, 9, -2, 1, 0], 4], [[0, 10, -18, -8, -9, -2, -1, 0], 4], [[14, 0, 32, 0, 11, 0, 1, 0], 2], [[14, 56, 32, 95, 11, 32, 1, 3], 4]]

hecke_ring_character_values = [[271, [1, 0, 0, 0, 0, 0, 0, 0]], [541, [1, 0, 0, 0, 0, 0, 0, 0]], [1001, [0, -1, 0, 0, 0, 0, 0, 0]], [217, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, -1, 0, 0, 0, 1, -1], [0, -1, 0, 0, 0, -1, 0, -1], [-2, 0, 1, 1, 0, 0, 1, 0], [0, 0, -1, -1, 2, -2, -1, 0], [0, 2, 0, -1, 0, 0, 0, 0], [2, -2, 1, 0, -1, 0, 0, 0], [0, 1, 0, -2, 0, -1, 0, -1], [1, 0, 2, 2, -1, 1, 1, 0], [0, 2, 0, 0, 0, -1, 0, 1], [2, -2, 1, 0, 0, 0, 1, -1], [1, -1, 3, 0, -1, 0, 1, -1], [-4, 0, 0, 0, 0, 0, 2, 0], [0, 3, 0, 1, 0, -1, 0, -2], [-5, 5, -1, 0, 0, 0, 1, -1], [0, 2, 0, 0, 0, -2, 0, -1], [-3, 0, 0, 0, 1, -1, -1, 0], [3, 0, -1, -1, 1, -1, -2, 0], [4, -4, 0, 0, 2, 0, 2, -2], [6, -6, -1, 0, 2, 0, -2, 2], [-4, 0, 1, 1, -1, 1, -2, 0], [1, -1, -3, 0, -1, 0, 0, 0], [4, -4, 2, 0, 0, 0, 0, 0], [0, 7, 0, 2, 0, 1, 0, -1], [-1, 0, -2, -2, 1, -1, 2, 0], [-3, 0, -3, -3, 0, 0, 1, 0], [0, 4, 0, 2, 0, 2, 0, 2], [-11, 0, 1, 1, -1, 1, 3, 0], [0, 6, 0, -2, 0, 4, 0, 2], [5, -5, 1, 0, -3, 0, 2, -2], [0, 7, 0, -5, 0, -1, 0, -2], [0, 1, 0, 1, 0, 4, 0, 1], [5, -5, 0, 0, -1, 0, -1, 1], [0, 4, 0, 2, 0, 0, 0, 4], [-5, 0, -2, -2, -1, 1, -1, 0], [0, 5, 0, 1, 0, -1, 0, -1], [0, 0, -2, 0, 0, 0, 2, -2], [-5, 0, -2, -2, -1, 1, -1, 0], [12, 0, 1, 1, -1, 1, 0, 0], [9, -9, -4, 0, 3, 0, -3, 3], [0, -4, 0, -1, 0, 0, 0, -1], [-6, 0, 4, 4, 0, 0, 2, 0], [-3, 0, 0, 0, 1, -1, -3, 0], [0, -7, 0, 2, 0, 1, 0, 1], [5, -5, -7, 0, 1, 0, -1, 1], [-6, 6, -3, 0, 2, 0, 1, -1], [0, -10, 0, 5, 0, -1, 0, -2], [4, 0, -1, -1, -4, 4, 1, 0], [0, 8, 0, -2, 0, 2, 0, 2], [-14, 14, 0, 0, 1, 0, -1, 1], [-6, 0, -1, -1, -2, 2, 1, 0], [0, 7, 0, -3, 0, 1, 0, 2], [9, -9, -2, 0, 1, 0, -3, 3], [5, 0, -7, -7, 2, -2, -5, 0], [-5, 0, 0, 0, -1, 1, -1, 0], [-9, 9, 4, 0, -5, 0, 5, -5], [-1, 1, 3, 0, -2, 0, -3, 3], [0, -10, 0, 1, 0, 4, 0, 6], [0, -1, 0, -4, 0, -3, 0, -3], [-3, 0, 4, 4, -3, 3, 0, 0], [0, -5, 0, 2, 0, -1, 0, -3], [2, -2, -5, 0, 4, 0, 1, -1], [7, -7, 4, 0, -1, 0, 3, -3], [-22, 22, -2, 0, 2, 0, -2, 2], [0, -3, 0, 1, 0, 3, 0, 2], [0, -6, 0, 1, 0, -4, 0, 1], [0, 0, 1, 0, -3, 0, 4, -4], [-5, 5, 3, 0, 1, 0, -2, 2], [-17, 0, 0, 0, 1, -1, 3, 0], [25, -25, -2, 0, 1, 0, -3, 3], [0, 4, 0, -4, 0, -4, 0, -6], [-9, 0, -3, -3, 3, -3, 2, 0], [0, 11, 0, 2, 0, 5, 0, 3], [-7, 7, 1, 0, 6, 0, -3, 3], [-4, 0, 2, 2, 2, -2, 4, 0], [0, -1, 0, 9, 0, 3, 0, 2], [0, 3, 0, 3, 0, 5, 0, 0], [0, -1, 0, 2, 0, -1, 0, 3], [-8, 8, 0, 0, 2, 0, 2, -2], [12, 0, 2, 2, -6, 6, 4, 0], [-2, 0, 3, 3, 4, -4, -5, 0], [12, -12, 10, 0, -6, 0, 2, -2], [-6, 6, 4, 0, -4, 0, -1, 1], [11, 0, -5, -5, -3, 3, 0, 0], [12, -12, 1, 0, 4, 0, -5, 5], [-6, 6, 1, 0, 1, 0, 6, -6], [0, 17, 0, -2, 0, 1, 0, 1], [12, 0, -5, -5, -2, 2, 5, 0], [-10, 10, 0, 0, 0, 0, 4, -4], [-33, 0, 0, 0, 1, -1, 1, 0], [0, 5, 0, 1, 0, -5, 0, -2], [0, -5, 0, 1, 0, 1, 0, -4], [-5, 0, -3, -3, -5, 5, 9, 0], [0, -8, 0, -1, 0, 1, 0, 0], [15, 0, 2, 2, -4, 4, 4, 0], [7, 0, 3, 3, -3, 3, 3, 0], [-2, 0, 1, 1, 7, -7, -6, 0], [-7, 7, -2, 0, 1, 0, 4, -4], [0, 0, 6, 6, -2, 2, 4, 0], [0, -16, 0, 2, 0, -2, 0, -5], [-21, 21, -3, 0, 3, 0, -2, 2], [0, -11, 0, 5, 0, 1, 0, 2], [9, 0, -11, -11, 3, -3, -4, 0], [-25, 25, 6, 0, 3, 0, -4, 4], [-9, 0, 0, 0, 5, -5, 1, 0], [0, 4, 0, 2, 0, -2, 0, 4], [-15, 15, 9, 0, -3, 0, 4, -4], [0, 7, 0, -7, 0, -1, 0, -1], [11, 0, 0, 0, 3, -3, 5, 0], [0, 9, 0, 11, 0, 3, 0, -4], [-6, 6, -7, 0, 4, 0, 3, -3], [8, 0, 2, 2, -2, 2, 2, 0], [-6, 6, 0, 0, -5, 0, 5, -5], [0, 0, 0, 0, 0, 2, 0, -1], [31, 0, -1, -1, 3, -3, -3, 0], [0, -1, 0, -4, 0, -1, 0, 1], [14, -14, 4, 0, -8, 0, 2, -2], [0, -32, 0, -2, 0, -2, 0, 0], [3, -3, -2, 0, -1, 0, 5, -5], [-5, 5, -4, 0, -1, 0, 3, -3], [12, 0, 1, 1, -2, 2, 3, 0], [-4, 4, -2, 0, -2, 0, 4, -4], [-3, 0, 7, 7, -2, 2, -1, 0], [-6, 6, -5, 0, 1, 0, -4, 4], [0, 0, -6, -6, 4, -4, 0, 0], [5, -5, -3, 0, 3, 0, 1, -1], [0, 16, 0, 6, 0, 2, 0, 0], [13, 0, -3, -3, -1, 1, 8, 0], [0, -2, 0, -5, 0, -2, 0, 0], [0, 1, 0, 2, 0, 7, 0, 1], [-19, 0, 2, 2, -1, 1, 3, 0], [0, -14, 0, -3, 0, -3, 0, 4], [0, -18, 0, 5, 0, 1, 0, 4], [26, 0, 2, 2, 2, -2, -4, 0], [0, 9, 0, -2, 0, 7, 0, 1], [0, 13, 0, 12, 0, 3, 0, 1], [17, 0, 7, 7, -1, 1, -2, 0], [15, 0, 7, 7, -5, 5, 4, 0], [-3, 3, -9, 0, 8, 0, -3, 3], [0, -13, 0, 7, 0, -5, 0, -1], [3, 0, 7, 7, -1, 1, -1, 0], [-13, 0, 1, 1, -8, 8, 7, 0], [3, -3, 2, 0, -5, 0, 5, -5], [9, -9, 4, 0, -7, 0, 1, -1], [19, -19, -2, 0, 9, 0, -7, 7], [0, -6, 0, -1, 0, 4, 0, 11], [-7, 0, -3, -3, 1, -1, -5, 0], [0, -1, 0, -2, 0, -1, 0, -3], [-5, 0, 5, 5, -6, 6, 9, 0], [12, 0, -6, -6, 6, -6, -5, 0], [0, -33, 0, 0, 0, 1, 0, -3], [10, -10, 4, 0, 0, 0, -1, 1], [-8, 8, -4, 0, 4, 0, -2, 2], [-22, 0, 4, 4, -8, 8, 12, 0], [8, -8, 8, 0, -2, 0, 2, -2], [-16, 0, -2, -2, 2, -2, -6, 0], [0, -4, 0, -7, 0, 1, 0, -8], [16, -16, -6, 0, 8, 0, -11, 11], [24, 0, -5, -5, -2, 2, 3, 0], [0, 10, 0, -9, 0, -4, 0, 0], [-3, 0, -12, -12, 5, -5, -3, 0], [0, -2, 0, -7, 0, 4, 0, -1], [-6, 6, 1, 0, 8, 0, 2, -2], [6, 0, 6, 6, 2, -2, 2, 0], [8, -8, -12, 0, 0, 0, 2, -2], [-4, 0, 3, 3, -5, 5, 2, 0], [0, -3, 0, 4, 0, 1, 0, 3], [-15, 15, 5, 0, -3, 0, 6, -6], [0, 4, 0, 3, 0, 5, 0, 4], [0, 19, 0, 4, 0, 3, 0, -3], [-7, 7, 9, 0, -3, 0, -2, 2], [0, -10, 0, -6, 0, 2, 0, 0], [0, 4, 0, -8, 0, 4, 0, 6], [-27, 27, -3, 0, -3, 0, -2, 2], [18, 0, -2, -2, 2, -2, -2, 0], [22, 0, 1, 1, -4, 4, 2, 0], [19, -19, 3, 0, -2, 0, -5, 5], [13, -13, -6, 0, 5, 0, 1, -1], [16, -16, -1, 0, 4, 0, -1, 1], [0, -17, 0, 2, 0, 1, 0, 3], [-2, 0, -3, -3, 2, -2, -9, 0], [0, -3, 0, 9, 0, 9, 0, 1], [-32, 32, 1, 0, -1, 0, -2, 2], [-10, 0, 10, 10, 6, -6, -8, 0], [10, -10, 1, 0, 6, 0, -2, 2], [0, 14, 0, 6, 0, -1, 0, 3], [4, 0, -8, -8, -4, 4, -2, 0], [12, 0, 7, 7, -10, 10, 3, 0], [20, -20, -2, 0, -4, 0, 7, -7], [16, 0, -5, -5, 8, -8, -5, 0], [15, -15, -5, 0, 8, 0, -9, 9], [2, 0, -8, -8, 0, 0, -5, 0], [0, -6, 0, 3, 0, -6, 0, -11], [0, -22, 0, -10, 0, -5, 0, 5], [18, -18, -10, 0, 0, 0, -4, 4], [6, -6, -3, 0, 10, 0, -3, 3], [-18, 0, 15, 15, 4, -4, -2, 0], [0, 24, 0, -2, 0, -6, 0, 4], [-1, 1, -6, 0, 3, 0, 3, -3], [0, 32, 0, -8, 0, 8, 0, 4], [-11, 11, -2, 0, 0, 0, -4, 4], [-31, 0, 3, 3, -1, 1, 0, 0], [-3, 0, 8, 8, -5, 5, -3, 0], [7, 0, -6, -6, 1, -1, -1, 0], [0, 1, 0, -14, 0, -5, 0, -4], [-15, 15, 2, 0, -8, 0, 10, -10], [0, -21, 0, -9, 0, -3, 0, 2], [-23, 0, -5, -5, -3, 3, -2, 0], [0, -6, 0, 4, 0, 4, 0, 0], [-23, 23, 13, 0, 1, 0, 3, -3], [-7, 7, -10, 0, 1, 0, 9, -9], [0, 13, 0, -12, 0, -3, 0, 3], [-2, 2, 0, 0, -7, 0, 11, -11], [0, 18, 0, 3, 0, 4, 0, -4], [-5, 5, 13, 0, -9, 0, 6, -6], [5, 0, 3, 3, 3, -3, -1, 0], [0, -8, 0, -10, 0, -6, 0, 2], [0, -5, 0, 7, 0, 12, 0, 1], [0, 13, 0, 12, 0, 1, 0, 3], [0, -15, 0, -4, 0, -4, 0, 0], [-10, 0, -9, -9, -2, 2, 4, 0], [0, 2, 0, 14, 0, 10, 0, 11], [2, -2, 18, 0, -6, 0, 6, -6], [-7, 7, -4, 0, -1, 0, 3, -3], [-43, 0, 0, 0, 5, -5, -1, 0], [1, -1, -1, 0, 5, 0, 3, -3], [15, -15, 4, 0, 3, 0, -1, 1], [0, -8, 0, 4, 0, -8, 0, -2], [-5, 0, -6, -6, 1, -1, -9, 0], [0, -3, 0, 4, 0, -1, 0, -1], [0, -4, 0, 2, 0, -9, 0, -3], [-23, 23, 1, 0, -3, 0, 5, -5], [-14, 14, 1, 0, 10, 0, -8, 8], [0, 9, 0, -5, 0, -6, 0, -5], [21, 0, 8, 8, -9, 9, 5, 0], [0, 14, 0, -1, 0, -4, 0, 5], [9, 0, -6, -6, 11, -11, 1, 0], [-25, 25, 8, 0, -7, 0, 4, -4], [25, 0, 10, 10, -3, 3, 1, 0], [0, 7, 0, -3, 0, 9, 0, 5], [10, 0, -7, -7, 7, -7, -8, 0], [0, 28, 0, 9, 0, 2, 0, -5], [-11, 11, -12, 0, -1, 0, -3, 3], [-12, 0, -5, -5, 6, -6, 0, 0], [0, 38, 0, 1, 0, 4, 0, -1], [0, -27, 0, -8, 0, -11, 0, -3], [-32, 0, 5, 5, 4, -4, -2, 0], [0, 19, 0, -4, 0, -5, 0, 1], [11, 0, 4, 4, 0, 0, -2, 0], [0, -13, 0, -4, 0, -9, 0, 1], [31, -31, 7, 0, -4, 0, 5, -5], [26, -26, -8, 0, 10, 0, -6, 6], [-3, 0, -9, -9, -1, 1, 4, 0], [-20, 0, -8, -8, 4, -4, -10, 0], [-29, 29, -1, 0, -3, 0, -9, 9], [-31, 0, 2, 2, 7, -7, -3, 0], [0, 0, -1, -1, 2, -2, -3, 0], [17, -17, -3, 0, 1, 0, 11, -11], [-8, 8, 0, 0, -8, 0, 7, -7], [0, 2, 0, -8, 0, -6, 0, -12], [8, 0, 4, 4, 0, 0, -2, 0], [0, 9, 0, -2, 0, -7, 0, -17], [-9, 9, 1, 0, -5, 0, -2, 2], [-34, 0, 0, 0, -6, 6, 12, 0], [27, -27, 12, 0, -2, 0, -4, 4], [0, 24, 0, -2, 0, -2, 0, 5], [0, 12, 0, 14, 0, 8, 0, 0], [0, 0, 0, -4, 0, 8, 0, -6], [36, 0, -7, -7, 2, -2, -4, 0], [30, -30, 3, 0, -4, 0, -5, 5], [0, 10, 0, -8, 0, 6, 0, 4], [0, -7, 0, -11, 0, -7, 0, -2], [-39, 0, -5, -5, 9, -9, -1, 0], [0, 27, 0, -3, 0, -9, 0, -1], [-12, 12, -2, 0, -4, 0, 16, -16], [-28, 0, 10, 10, 7, -7, -3, 0], [26, -26, 7, 0, 4, 0, -1, 1], [0, 3, 0, 4, 0, 5, 0, 9], [0, -7, 0, -8, 0, -1, 0, 11], [-15, 15, 6, 0, -9, 0, 1, -1], [-28, 28, 13, 0, 3, 0, -2, 2], [0, -9, 0, -6, 0, -9, 0, -2], [-18, 0, 6, 6, 2, -2, -4, 0], [7, 0, 2, 2, -1, 1, 5, 0], [0, 46, 0, 0, 0, -2, 0, 2], [-12, 12, 0, 0, 0, 0, -12, 12], [8, 0, 0, 0, 9, -9, -1, 0], [16, -16, -8, 0, 0, 0, -2, 2], [-4, 0, -7, -7, 6, -6, -5, 0], [0, -4, 0, -7, 0, 8, 0, 3], [0, 0, 0, 11, 0, 0, 0, -1], [11, -11, 4, 0, -11, 0, 9, -9], [0, -57, 0, 3, 0, 6, 0, 9], [3, -3, -8, 0, 3, 0, -3, 3], [6, -6, -6, 0, -4, 0, 4, -4], [-28, 0, -6, -6, 6, -6, 0, 0], [-3, 3, -10, 0, 5, 0, -3, 3], [0, -22, 0, 1, 0, 6, 0, 9], [-38, 0, 2, 2, 4, -4, -4, 0], [-17, 0, 0, 0, -3, 3, -13, 0]]
